import java.math.*;

/**
 * <p> Autor Michael Gorski </p>
 * <p> Version 1.0 </p>
 * <p>Copyright: Copyright (c) 2004 </p>
 */

/** Klasse zum speichern beliebig vieler Elemente vom Typ ProjectivePoint in einer
BigProjectivePointArray Datenstruktur. */
public class BigProjectivePointArray
{
	final static BigInteger zero = new BigInteger("0");
	final static BigInteger one = new BigInteger("1");
	final static BigInteger two = new BigInteger("2");
	final static BigInteger three = new BigInteger("3");
	final static BigInteger four = new BigInteger("4");
	final static BigInteger five = new BigInteger("5");
	final static BigInteger six = new BigInteger("6");
	final static BigInteger seven = new BigInteger("7");
	final static BigInteger eight = new BigInteger("8");

	class Node
	{
		public ProjectivePoint [] data;
		public BigInteger number;
		public Node next;

		Node (ProjectivePoint [] d, BigInteger num, Node n)
		{
			number = num;
			data = d;
			next = n;
		}
	}  // end Node
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    final static BigInteger nodeLength	= new BigInteger("2000000");// Anzahl der Elemente pro Knoten
   	BigInteger length = zero;									// Anzahl der Elemente insgesamt
   	BigInteger countNodes = zero;								// Anzahl der Knoten
    BigInteger lastNodeNumber = zero;							//Nummer des letzten Knotens

   	Node start = null;

//======================================================================================================

	/** Konstruktor, um ein BigProjectivePointArray fr maximal l Elemente vom Typ ProjectivePoint
	zu erzeugen. */
   	public BigProjectivePointArray(BigInteger l)
   	{
	   	length = l;
	   	countNodes = length.divide(nodeLength);
	   	if ( !((length.mod(nodeLength)).compareTo(zero) == 0) )
	   		countNodes = countNodes.add(one);

		for(BigInteger i = zero; i.compareTo(countNodes) == -1;i = i.add(one))
			insert(new ProjectivePoint[(this.nodeLength).intValue()]);
   	}  // end BigArray

//======================================================================================================

   	private void insert(ProjectivePoint [] o)
   	{

   	   	if (start == null)
   	   	{
   	       	start      = new Node(o, lastNodeNumber, null);
   	       	start.next = start;
	   	}
		else
		{
			start = getNode(lastNodeNumber.subtract(one));
        	start.next = new Node(o, lastNodeNumber, start.next);
		}
		lastNodeNumber = lastNodeNumber.add(one);
   	}  // end insert

//======================================================================================================

   	private Node getNode(BigInteger i)
   	{
		while(true)
		{
			if ( (start.number).compareTo(i) == 0 )
				return(start);
			else
			    start = start.next;
		} // end while

   	} //end getNodeNumber

//======================================================================================================

	/** Fgt ein ProjectivePoint in das BigProjectivePointArray an Position pos ein. */
	public void set(BigInteger pos, ProjectivePoint elem)
	{
		// entscheiden in welchem Knoten elem liegt
		BigInteger nodeNumber = pos.divide(nodeLength);
		// entscheiden auf welcher Position im Knoten elem liegt
		BigInteger nodePlace = pos.mod(nodeLength);
		Node n = getNode(nodeNumber);
		n.data[nodePlace.intValue()] = elem;
	} // end set

//======================================================================================================

	/** Gibt aus dem BigProjectivePointArray den an Position pos liegenden ProjectivePoint zurck. */
	public ProjectivePoint get(BigInteger pos)
	{
		// entscheiden in welchem Knoten elem liegt
		BigInteger nodeNumber = pos.divide(nodeLength);
		// entscheiden auf welcher Position im Knoten elem liegt
		BigInteger nodePlace = pos.mod(nodeLength);
		Node n = getNode(nodeNumber);

		if ( (n.data[nodePlace.intValue()]) == null )
			return(new ProjectivePoint());

		return(n.data[nodePlace.intValue()]);
	} // end get

//======================================================================================================

	/** Gibt die Gre des BigProjectivePointArrays zurck. */
	public BigInteger getArrayLength()
	{
		return(length);
	} // end getArrayLength

//======================================================================================================
} // end BigProjectivePointArray
