import java.math.*;

/**
 * <p> Autor Michael Gorski </p>
 * <p> Version 1.0 </p>
 * <p>Copyright: Copyright (c) 2004 </p>
 */

/**Klasse, um elliptische Kurven darstellen und mit ihnen rechnen zu knnen.*/
public class EllipticCurve  //y^2 = x^3 +ax + b
{
	BigInteger a;
	BigInteger b;
	BigInteger p;

	BigInteger eee = new BigInteger("0");

	final static BigInteger zero = new BigInteger("0");
	final static BigInteger one = new BigInteger("1");
	final static BigInteger two = new BigInteger("2");
	final static BigInteger three = new BigInteger("3");
	final static BigInteger four = new BigInteger("4");
	final static BigInteger five = new BigInteger("5");
	final static BigInteger six = new BigInteger("6");
	final static BigInteger seven = new BigInteger("7");
	final static BigInteger eight = new BigInteger("8");

//======================================================================================================

	/** Konstruktor, um eine elliptische Kurve aus den Kurvenparametern a, b, p zu erzeugen.
	    Die Parameter a und b sind hierbei die Kurvenparameter von y^2 = x^3 + ax + b. Der Parameter p
	    gibt die Anzahl der Elemente des betrachteten Krpers F_p an.
	    Alle Parameter werden hierbei als BigInteger Werte bergeben.*/
	public EllipticCurve(BigInteger a, BigInteger b, BigInteger p)
	{
		setVariables(a,b,p);
	}

//======================================================================================================

	/** Konstruktor, um eine elliptische Kurve aus den Kurvenparametern a, b, p zu erzeugen.
	    Die Parameter a und b sind hierbei die Kurvenparameter von y^2 = x^3 + ax + b.Der Parameter p
	    gibt die Anzahl der Elemente des betrachteten Krpers F_p an.
	    Alle Parameter werden hierbei als String Werte bergeben.*/
	public EllipticCurve(String a, String b, String p)
	{
		setVariables(new BigInteger(a),new BigInteger(b),new BigInteger(p));
	}

//======================================================================================================

	/** Konstruktor, um eine elliptische Kurve aus a,x,y,p oder aus b,x,y,p zu erzeugen.
	    Der Parameter typ kann entweder "axyp" oder "bxyp" sein. Er gibt an, ob zustzlich zu einem
	    affinen Kurvenpunkt (x,y) der Kurvenparameter a oder b in param bergeben wird.
		Der Parameter param ist also entweder der Parameter a oder b der Kurve y^2 = x^3 + ax + b.
		Der Parameter p gibt die Anzahl der Elemente des betrachteten Krpers F_p an. */
	public EllipticCurve(String typ, BigInteger param, BigInteger x, BigInteger y, BigInteger p)
	{
		BigInteger a = zero;
		BigInteger b = zero;
		BigInteger temp1, temp2,temp3;
		BigInteger value2 = x;
		BigInteger value3 = y;

		if (typ == "axyp")
		{
			a = param;
			temp1 = value3.modPow(two,p);										// y^2
			temp2 = value2.modPow(three,p);										// x^3
			temp3 = (param.multiply(value2) ).mod(p);							// ax

			b = ( (( temp1.subtract(temp2) ).mod(p)).subtract(temp3) ).mod(p);	// y^2-x^3-ax = b
		}
		else if (typ == "bxyp")
		{
			b = param;
			temp1 = value3.modPow(two,p);										// y^2
			temp2 = value2.modPow(three,p);										// x^3
			temp3 = temp2.modInverse(p);										// x^{-1}

			// (y^2-x^3-b) x^{-1} = a
			a = ( (( (( temp1.subtract(temp2) ).mod(p)).subtract(b) ).mod(p)).multiply(temp3) ).mod(p);
		}
		setVariables(a,b,p);
	}

//======================================================================================================

	private BigInteger getB()
	{
		return(this.b);
	} // end getB

//======================================================================================================

	private void setVariables(BigInteger a, BigInteger b, BigInteger p)
	{
		// testet, ob p eine Primzahl ist
		if ( !(p.isProbablePrime(2000)) )		//testet, ob p eine primzahl ist
		{
			MathFunction.print("[setVariables]: p ist keine Primzahl.");
			p = p.nextProbablePrime();
			MathFunction.print("[setVariables]: p wurde auf die naechst groessere Primzahl gesetzt.");
			MathFunction.print("[setVariables]: p = "+ p.toString() );
		}
		MathFunction.print("[setVariables]: p ist: "+ p.bitLength() +" Bit lang.\n\n");
		this.p = p;
		this.a = a.mod(p);
		this.b = b.mod(p);

		BigInteger test, test1, test2;
		boolean found = false;

		// testet, ob 4a^3 + 27b^2 <> 0 mod p ist

		do
		{
			test1 = ( ((this.a).modPow(three,p)).multiply(four) ).mod(p);				// 4a^3 mod p
			test2 = ( ((this.b).modPow(two,p)).multiply(new BigInteger("27")) ).mod(p); //	27b^2

			test = ( test1.add(test2) ).mod(p);
			if (test. compareTo(zero) == 0 )
			{
				MathFunction.print("[setVariables]: 4a^3 + 27b^2 = 0");
				this.b = (this.b).add(one);
				MathFunction.print("[setVariables]: b wurde auf "+this.b+" gesetzt!");
			}
			else
				found = true;

		} while(!found);
	}  // end setVariables

//======================================================================================================

	// konvertiert ein BigInteger k in einen NAF String
	private static int[] bigIntToNAF(BigInteger k)
	{
		int[] NAF = new int [getNAFLength(k)];     	// for NAF - Form
		BigInteger c = zero;
		c = k;
		int l = NAF.length - 1;
		while( c.compareTo( new BigInteger(""+0)) == 1 )
		{
			if( ( c.mod(new BigInteger(""+2) ).compareTo(one) == 0 )) //if c = 1 mod 2
			{
				NAF[l]= 2 - (( c.mod(four) ).intValue());  	//NAF[l] <- 2 - (c mod 4)
                c = c.subtract(new BigInteger(""+NAF[l]));   				//c <- c - NAF[l]
			} // end if
			else
			  NAF[l] = 0;

			c = c.divide(two);								// c <- c/2
			l = l - 1;                        								// l <- l - 1
		} // end while
		return(NAF);
	} // end bigIntToNAF

//======================================================================================================

    //berechnet die Lnge der NAF
	private static int getNAFLength(BigInteger k)
    {
		int length = 0;
		String A = k.toString(2);                             		// the BigInteger k into binary Form
		length = ((A.toCharArray()).length)+1;
		return(length);
	} // end getNAFLength

//======================================================================================================
//   public Teil																					 ===
//======================================================================================================


	/** berprft, ob zwei projektive Punkte dem gleichen affinen Punkt entsprechen. */
	public boolean isEqual(ProjectivePoint P, ProjectivePoint Q)
	{
		boolean test = false;
		BigInteger x1 = zero;
		BigInteger x2 = zero;
        BigInteger y1 = zero;
        BigInteger y2 = zero;
        BigInteger z1 = zero;
        BigInteger z2 = zero;

		if(  ( (P.getZ()).compareTo(new BigInteger("0")) == 0 )&&   // P,Q = 0 (infinity Point)
		     ( (Q.getZ()).compareTo(new BigInteger("0")) == 0 ))
		{
			test = true;
	    } // end if
		else if( ( (P.getZ()).compareTo(new BigInteger("0")) == 0 )&&   // P = 0, Q <> 0
		     	 ( (Q.getZ()).compareTo(new BigInteger("0")) != 0 )  )
		{
			test = false;
	    } // end if
		else if(  ( (P.getZ()).compareTo(new BigInteger("0")) != 0 )&&   // P <> 0,  Q = 0
		          ( (Q.getZ()).compareTo(new BigInteger("0")) == 0 )  )
		{
			test = false;
	    } // end if
	    else if(  ( (P.getZ()).compareTo(new BigInteger("0")) != 0 )&&   // P,Q <> 0
		          ( (Q.getZ()).compareTo(new BigInteger("0")) != 0 ) )
		{
			x1 = ( (P.getX()).multiply( Q.getZ() ) ).mod(p);

			y1 = ( (P.getY()).multiply( Q.getZ() ) ).mod(p);

			z1 = ( (P.getZ()).multiply( Q.getZ() ) ).mod(p);


			x2 = ( (Q.getX()).multiply( P.getZ() ) ).mod(p);

			y2 = ( (Q.getY()).multiply( P.getZ() ) ).mod(p);

			z2 = ( (Q.getZ()).multiply( P.getZ() ) ).mod(p);

		    if ( (x1.compareTo(x2) == 0) && (y1.compareTo(y2) == 0) && (z1.compareTo(z2) == 0) )
	        {
				test = true;
			}
		}
		return(test);
	} // end isEqual

//======================================================================================================

	/** Addiert zwei projektive Punkte und liefert den resultierenden Punkt zurck. */
	public ProjectivePoint add(ProjectivePoint P, ProjectivePoint Q)
	{
		ProjectivePoint Z = new ProjectivePoint(); // for initialisation of Z

		BigInteger u1,u2,u,v1,v2,v,v_pow2,v_pow3, u_pow2, A1,A2,A3,A4,A5,A6,A,t1,t2,t3;
		BigInteger w1,w2,w,s,B,h1,h2,h3,h,t4,t5,t6,t7;

		BigInteger x1 = P.getX();
		BigInteger x2 = Q.getX();
		BigInteger y1 = P.getY();
		BigInteger y2 = Q.getY();
		BigInteger z1 = P.getZ();
		BigInteger z2 = Q.getZ();

		if ( (P.getZ()).compareTo(zero) == 0 ) 			// P = 0 (infinity Point)
		{
			Z.setX( Q.getX());
			Z.setY( Q.getY());
			Z.setZ( Q.getZ());
			return(Z);
		}
		else if ( (Q.getZ()).compareTo(zero) == 0 ) 		// Q = 0 (infinity Point)
		{
			Z.setX( P.getX());
			Z.setY( P.getY());
			Z.setZ( P.getZ());
			return(Z);
		}

        else if( isEqual(P,Q) )		// P,Q <> 0 (infinity Point) and P = Q  => P + Q = 2P
        {
			w1 = ( (z1.modPow(two,p)).multiply(a) ).mod(p);					// a z1^2
			w2 = ( (x1.modPow(two,p)).multiply(three) ).mod(p);				// 3 x1^2
			w = ( w1.add(w2) ).mod(p);

			s = ( y1.multiply(z1) ).mod(p);									// y1 z1

			B = ( (( x1.multiply(y1) ).mod(p)).multiply(s) ).mod(p);		// x1 y1 s

			h1 = w.modPow(two,p);											// w^2
			h2 = ( four.multiply(B) ).mod(p);								// 4B
			h3 = ( two.multiply(h2) ).mod(p);								// 8B
			h = ( h1.subtract(h3) ).mod(p);									// h1 - h2

			Z.setX(  ( (( two.multiply(h) ).mod(p)).multiply(s) ).mod(p)  );  // 2 h s

			t1 = ( h2.subtract(h) ).mod(p);									// 4B - h

			t2 = y1.modPow(two,p);											// y1^2
			t3 = s.modPow(two,p);											// s^2

			t4 = ( w.multiply(t1) ).mod(p);									// w( 4B - h )
			t5 = ( t2.multiply(t3) ).mod(p);								// t2 t3
			t6 = ( eight.multiply(t5) ).mod(p);								// 8 t2 t3

			Z.setY(  ( t4.subtract(t6) ).mod(p)  );							// t4 - t6

			t7 = ( s.multiply(t3) ).mod(p);									// s^3

			Z.setZ(  ( eight.multiply(t7) ).mod(p)  );						// 8 s^3

		} // end else if
		else if ( !( isEqual(P,Q) )&&
		     ((P.getZ()).compareTo(zero) != 0 )&&	      // P <> 0 (infinity Point)
		     ((Q.getZ()).compareTo(zero) != 0 ))	      // Q <> 0 (infinity Point)
	  	{
			u1 = ( y2.multiply(z1) ).mod(p);
			u2 = ( y1.multiply(z2) ).mod(p);
			u = ( u1.subtract(u2) ).mod(p);						// u = y_2 z_1 - y_1 z_2

			v1 = ( x2.multiply(z1) ).mod(p);
			v2 = ( x1.multiply(z2) ).mod(p);
			v = ( v1.subtract(v2) ).mod(p);						// v = x_2 z_1 - x_1 z_2

			v_pow2 = v.modPow(two,p);										// v^2
			v_pow3 = ( v_pow2.multiply(v) ).mod(p);							// v^3
			u_pow2 = u.modPow(two,p);										// u^2

			A1 = ( z1.multiply(z2) ).mod(p);								// z1 z2
			A2 = ( x1.multiply(z2) ).mod(p);								// x1 z2
			A3 = ( u_pow2.multiply(A1) ).mod(p);							// u^2 z1 z2
			A4 = ( v_pow2.multiply(A2) ).mod(p);							// v^2 x1 z2
			A5 = ( two.multiply(A4) ).mod(p);								// 2 v^2 x1 z2
			A6 = ( A3.subtract(v_pow3) ).mod(p);							// A3 - v^3

			A = ( A6.subtract(A5) ).mod(p);									// A6 - A5

			Z.setX( ( v.multiply(A) ).mod(p) );

			t1 = ( v_pow3.multiply(u2) ).mod(p);							// v^3 y1 z2
			t2 = ( A4. subtract(A) ).mod(p);								// v^2 x1 z2 - A
			t3 = ( u.multiply(t2) ).mod(p);									// u( v^2 x1 z2 - A )

			Z.setY(  ( t3.subtract(t1) ).mod(p)   );

			Z.setZ(  ( v_pow3.multiply(A1) ).mod(p)  );

        } // end else if
		else if ( isEqual(P,negate(Q)) )			  // P == - Q
		{
			return( add(P,negate(Q)) );
		}
        return(Z);
	} // end add


//======================================================================================================

	/** Gibt einen projektiven Punkt in affiner Form als String zurck. */
	public String toAffineString( ProjectivePoint P)
	{
		String affine = "";

		if ( (P.getZ()).compareTo(zero) == 0 ) 			// P = 0 (z = 0 )
		{
			affine = "O";
			return affine;
		}
		affine += "(";
		affine += (  (((P.getX()).mod(p)).multiply( ( P.getZ() ).modInverse(p) )).mod(p)
                 ).toString();

		affine +=",";

		affine += (  (((P.getY()).mod(p)).multiply( ( P.getZ() ).modInverse(p) )).mod(p)
                 ).toString();
		affine +=")";
		return(affine);
	} // end toAffineString

//======================================================================================================

    /** Negiert den bergebenen projektiven Punkt und gibt ihn zurck. */
    public ProjectivePoint negate(ProjectivePoint P)
    {
		ProjectivePoint N = new ProjectivePoint();
        N.setX( P.getX());
        N.setY( ( (P.getY()).negate() ).mod(p) );
		N.setZ( P.getZ());

        return(N);
	}
//======================================================================================================

    /** Berechnet die Subtraktion zweier projektiver Punkte und liefert den resultierenden Punkt
    zurck. */
    public ProjectivePoint sub(ProjectivePoint P, ProjectivePoint Q)
    {
		return( add(P,negate(Q)) );

	} // end sub

//======================================================================================================

	/** Berechnet die skalare Multiplikation eines Punktes mit einem Skalar.
	Das Skalar k wird hierbei als integer bergeben. */
	public ProjectivePoint mul(int k, ProjectivePoint P)
	{
		return(mul(new BigInteger(""+k),P));
	} // end mul

//======================================================================================================

	/** Berechnet die skalare Multiplikation eines Punktes mit einem Skalar.
	Das Skalar k wird hierbei als String bergeben. */
	public ProjectivePoint mul(String k, ProjectivePoint P)
	{
		return(mul(new BigInteger(k),P));
	} // end mul

//======================================================================================================

	/** Berechnet die skalare Multiplikation eines Punktes mit einem Skalar.
	Das Skalar k wird hierbei als BigInteger bergeben. */
	public ProjectivePoint mul(BigInteger k, ProjectivePoint P)
	{
		ProjectivePoint Q = new ProjectivePoint(); // Z = 0
		int[] NAF = bigIntToNAF(k);
        for(int i = 0; i < NAF.length; i++)
        {
			Q = add(Q,Q);
			if (NAF[i] == 1)
			{
				Q = add(Q,P);
			} //end if
			else if (NAF[i] == -1)
			{
				Q = add(Q,negate(P));
			} // end else
		} //end for
        return(Q);
	} // end mul

//======================================================================================================

    /** berprft, ob ein Punkt auf der vorliegenden elliptischen Kurve liegt. */
    public boolean isCurvePoint(ProjectivePoint P)
    {
		boolean answer = false;
		BigInteger x = new BigInteger("0");
		BigInteger y = new BigInteger("0");
		BigInteger z = new BigInteger("0");
		BigInteger xt = new BigInteger("0");
		BigInteger yt = new BigInteger("0");
		x = P.getX();
		y = P.getY();
		z = P.getZ();
		//xt = x^3 + axz^2 + bz^3

		xt = (( ( ( x.modPow(new BigInteger("3"),p) )
		       ).add
		       ( ( ( ( ( z.modPow(new BigInteger("2"),p) ).multiply(x) ).mod(p) ).multiply(a) ).mod(p)
			   )
			 ).add
			 ( ( ( z.modPow(new BigInteger("3"),p) ).multiply(b) ).mod(p)
			 )).mod(p);
	    yt = ( (y.modPow(new BigInteger("2"),p) ).multiply(z) ).mod(p);

        if ( xt.compareTo(yt) == 0 )
          answer = true;
		return(answer);
	} // end isCurvePoint

//=====================================================================================================

    /** Bestimmt zufllig einen Punkt der betrachteten elliptischen Kurve. Es knnen alle Punkte
    auer der Punkt im Unendlichen zurckgegeben werden. */
	public ProjectivePoint getRandomPoint()
	{
		MathFunction.print("zufaelligen Punkt bestimmen");

		ProjectivePoint P = new ProjectivePoint();
		BigInteger x = zero;
		BigInteger yt = zero;
		BigInteger y = zero;
		boolean found = false;
		boolean found_LegendreZero = false;
		double rand = Math.random();

		while(!found)
		{
			rand = Math.random();

			x = MathFunction.getRandomNumberOdd(p);
		    yt = ( ( ( x.modPow(three,p) ).add
		           ( (x.multiply(a)).mod(p) )
		           ).add(b) ).mod(p);
		    if (MathFunction.getLegendre(yt,p) == 1)
				found = true;
		    if (MathFunction.getLegendre(yt,p) == 0)
		    {
				if (rand < 0.5) // um alle mglichen Punkte auer O gleichwahrscheinlich zu machen
				{
					found = true;
					found_LegendreZero = true;
				}
			}
		} //end while
		MathFunction.print("[getRandomPoint] x - Koordinate gefunden");

		P.setX(x);

		if (found_LegendreZero)
			P.setY(zero);
		else
		{
			y = MathFunction.getSqrt(yt,p);

			if (rand > 0.5)
			{
				y = ( y.negate() ).mod(p);
			}

			P.setY(y);
		}

		P.setZ(one);
		MathFunction.print("[getRandomPoint] ---> Punkt gefunden");
		return(P);
	} // end getRandomPoint

//======================================================================================================

//------------------------------------------------------------------------------------------------------
//  					                 Der  naive Algorithmus
//------------------------------------------------------------------------------------------------------

    /** Zhlt die Anzahl der Punkte auf der betrachteten elliptischen Kurve mit dem naiven Algorithmus.*/
	public BigInteger getOrder_ByNaive()
	{
		BigInteger order = one;
		BigInteger right = zero;
		int legendre = 0;

		MathFunction.print("\n----------------------------------------------------------------");
		MathFunction.print("\n-                  Der Naive Algorithmus                       -");
		MathFunction.print("\n----------------------------------------------------------------");

		for(BigInteger x = zero; x.compareTo(p) == -1; x = x.add(one))
		{
			right =(  (  ( x.modPow(three,p) ).add( (a.multiply(x)).mod(p) )  ).add(b)  ).mod(p);
			legendre = MathFunction.getLegendre(right, p);
			legendre++;
			order = order.add( new BigInteger(""+legendre) );
		}
		return(order);
	} // end getOrder_ByNaive

//=====================================================================================================

//------------------------------------------------------------------------------------------------------
//  				          	Der Algorithmus von Shanks Mestre
//------------------------------------------------------------------------------------------------------

    /** Zhlt die Anzahl der Punkte auf der betrachteten elliptischen Kurve mit dem Shanks Mestre
    Algorithmus.*/
	public BigInteger getOrder_ByShanks()
	{
		ProjectivePoint Q = new ProjectivePoint();
		ProjectivePoint D = new ProjectivePoint();
		ProjectivePoint R = new ProjectivePoint();
		ProjectivePoint W = new ProjectivePoint();
		BigInteger m = new BigInteger("0");
		BigInteger order = new BigInteger("0");
		BigInteger t = new BigInteger("0");
		BigDecimal temp = new BigDecimal("1.0");
		boolean foundOrder = false;
		int equals = 0;

//SCHRITT 1:
		MathFunction.print("\n----------------------------------------------------------------");
		MathFunction.print("\n-                  Der Algorithmus von Shanks Mestre           -");
		MathFunction.print("\n----------------------------------------------------------------");

//while(!foundOrder)
//{
		foundOrder = true;
		Q = getRandomPoint();
        MathFunction.print("[getOrder_ByShanks] Zufallspunkt Q: "+Q.toString()+" gefunden.");
        System.out.println("[getOrder_ByShanks] Zufallspunkt Q: "+Q.toString()+" gefunden.");

//SCHRITT 2:
		D = mul(p.add(one),Q); 					//D = (p+1) Q

		temp = (MathFunction.nSqrt(2,p)).multiply(new BigDecimal("2.0"));
        W = add(D, (mul(MathFunction.floor(temp),Q)) );				//W = D+ (2 floor(sqrt(p))) Q

//SCHRITT 3:
		temp = (MathFunction.nSqrt(4,p)).multiply(new BigDecimal("2.0"));
		m = MathFunction.ceil(temp);				//m = ceil(2 p^(1/4))
		R = mul(m,Q);

//SCHRITT 4:
		BigProjectivePointArray L1 = new BigProjectivePointArray(m);
		BigProjectivePointArray L2 = new BigProjectivePointArray(m);
		                                  // i < m
		for(BigInteger i = zero; i.compareTo(m) == -1; i = i.add(one))
		{
			L1.set( i, mul(i,Q) );

            //nur zum Test
			if( (i.mod(new BigInteger("500"))).compareTo(one) == 0)
			{
			   MathFunction.print("i: "+i.toString());
			   MathFunction.print("Bitlaenge von i: "+i.bitLength());
		    }
		} //end for i

		MathFunction.print("zweite Liste");

//SCHRITT 5:
		for(BigInteger j = zero; j.compareTo(m) == -1; j = j.add(one))
		{
			L2.set( j, add(W, negate( mul(j,R) ) ) );

            //nur zum Test
			if( (j.mod(new BigInteger("500"))).compareTo(one) == 0)
			{
			   MathFunction.print("j: "+j.toString());
			   MathFunction.print("Bitlaenge von j: "+j.bitLength());
		    }

			for(BigInteger i = zero; i.compareTo(m) == -1; i = i.add(one)) //+
			{
				//nur zum Test
				if( (i.mod(new BigInteger("1000"))).compareTo(one) == 0)
				{
			   		MathFunction.print("i: "+i.toString());
			   		MathFunction.print("Bitlaenge von i: "+i.bitLength());
			   		MathFunction.print("j: "+j.toString());
			   		MathFunction.print("Bitlaenge von j: "+j.bitLength());
		    	}

				if( isEqual( L1.get(i), L2.get(j) ) )
				{
					if ( equals > 1 )
					{
						foundOrder = false;
						break;
					}
					if ( equals == 0 )
					{
						MathFunction.print("i: "+i.toString());
						MathFunction.print("j: "+j.toString());
						temp = (MathFunction.nSqrt(2,p)).multiply(new BigDecimal("2.0"));
						t = ( ( j.multiply(m) ).add(i) ).subtract((MathFunction.floor(temp)));
						order = (p.add(one)).subtract(t);
						MathFunction.print("t: "+t.toString());
					}
					equals++;
				}
			} //end for i
		} //end for j
		if ( equals == 1 )
			MathFunction.print("\nDas Ergebis ist richtig!");
		else
			MathFunction.print("\nEs gibt mehr als eine Uebereinstimmung, das Ergebnis koennte Falsch sein!");

	//} // end while


		return(order);
	} // end getOrder_ByShanks

//======================================================================================================

//------------------------------------------------------------------------------------------------------
//  				            	Der Algorithmus von Schoof
//------------------------------------------------------------------------------------------------------

    /** Zhlt die Anzahl der Punkte auf der betrachteten elliptischen Kurve mit dem Schoof
    Algorithmus.*/
	public BigInteger getOrder_BySchoof()
	{
		BigDecimal temp;
		BigInteger sqrt;
		BigInteger order = zero;
		BigInteger l;										// aktuell verwendete Primzahl
		BigInteger l_n = two;								// grte verwendete Primzahl
		BigIntegerArray t_l;								// Array mit den Resten t_l
		BigInteger numberOfPrimes = one;
		BigInteger productOfPrimes;
		BigInteger counter = zero;							// um die Reste t_l an der korrekten
															// Position in t abspeichern zu knnen
		Polynom f_l;
		BigInteger pl;										// pl = p mod l
		Polynom ggT;										// fr den ggT von zwei Polynomen
		int legendre = 0;									// fr das Legendre Symbol
		Polynom gx;
		Polynom hx;
		BigInteger w = zero;								// w^2 = pl mod l
		Polynom k;
		BigInteger rl;										// Zwischenspeicher fr t_l
		Polynom Dx;
		Polynom Dy;
		BigInteger t;


//SCHRITT 1:
		MathFunction.print("\n----------------------------------------------------------------");
		MathFunction.print("\n-                  Der Algorithmus von Schoof                  -");
		MathFunction.print("\n----------------------------------------------------------------");

		temp = (MathFunction.nSqrt(2,p)).multiply(new BigDecimal("4.0"));
		sqrt = MathFunction.ceil(temp);						// sqrt = ceil(2 p^(1/4))
															// Schranke fr das Produkt der Primzahlen

		productOfPrimes = l_n;								// Produkt der Primzahlen
		while ( productOfPrimes.compareTo(sqrt) <= 0 )		// Produkt von i=2 bis l_n >= 4sqrt(p)
		{
			l_n = l_n.nextProbablePrime();					// nchste Primzahl
			numberOfPrimes = numberOfPrimes.add(one);		// Anzahl der bentigten Primzahlen (n)
			productOfPrimes = productOfPrimes.multiply(l_n);
		}  // end while

		t_l = new BigIntegerArray(numberOfPrimes);			// speichert die Reste t_l ab

		MathFunction.print("\n[SCHOOF]: benoetigten Primzahlen berechnet !");
		MathFunction.print("[SCHOOF]: groesste Primzahl fuer die Berechnung: "+l_n);
		MathFunction.print("[SCHOOF]: Anzahl der Primzahl fuer die Berechnung: "+numberOfPrimes);

//SCHRITT 2:

		l = two;

//SCHRITT 3:  t_l = #E(F_p) mod 2

		MathFunction.print("[SCHOOF]:aktueller Werte fuer l = " + l);

		Polynom poly1 = new Polynom(three);					// poly1 = x^3 + ax + b
		poly1.set(zero,one);
		poly1.set(two,a);
		poly1.set(three,b);

		Polynom poly2 = new Polynom(p);						// poly2 = x^p - x
		poly2.set(zero,one);
		poly2.set( p.subtract(one) ,new BigInteger("-1"));

		ggT = Polynom.ggT(poly1,poly2,p);					// ggT(poly1, poly2)

		if( ( ggT.getDeg() ).compareTo(zero) == 1 )			// deg( ggT(poly1, poly2) ) > 0
			t_l.set(counter,zero);							// t_l = 0
		else
			t_l.set(counter,one);							// t_l = 1


		MathFunction.print("[SCHOOF]:                               t_"+l+" = "+t_l.get(counter)+ " mod "+l);

//SCHRITT 4:
		l = l.nextProbablePrime();

		for(; l.compareTo(l_n) <= 0 ; l = l.nextProbablePrime()) 	// Falls l < l_n
		{
			counter = counter.add(one);

//SCHRITT 5: t_l = t mod l , l >= 3
			MathFunction.print(">>>>>>>>>>>>>>>>>>>><<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
			MathFunction.print("[SCHOOF]:aktueller Werte fuer l = " + l);

			pl = p.mod(l);											// pl = p mod l

			gx = Polynom.getGxHx("gx",pl,a,b,p,l);					// berechnet das Polynom g(x)
			MathFunction.print("[SCHOOF]: Polynom g(x) berechnet");

			f_l = Polynom.getDivisionPolynom(l,a,b,p);				// liefert das l-te reduzierte
           	           												// Divisionspolynom
           	MathFunction.print("[SCHOOF]: Polynom f_"+l+" berechnet");

			ggT = Polynom.ggT(gx, f_l,p);							// ggT(g(x), f_l)

			MathFunction.print("[SCHOOF]: ggTggT(gx, f_"+l+") berechnet");

			if( ( ggT.getDeg() ).compareTo(zero) == 1 )			// deg( ggT(g(x), f_l) ) > 0 (Fall A)
			{
				MathFunction.print("[SCHOOF]:                                                 Fall A");

				legendre = MathFunction.getLegendre(pl, l);			// Legendre von (pl/l)
				if ( legendre == 1 ) 								// Fall (pl/l) = 1 (Fall A1)
				{
					MathFunction.print("[SCHOOF]:                                                 Fall A1");
					w = MathFunction.getSqrt(pl, l);				// pl = w^2 mod l

					MathFunction.print("[SCHOOF]: Quadratwurzel von pl gefunden: w = "+w);

					hx = Polynom.getGxHx("hx",w,a,b,p,l);			// berechnet das Polynom h(x)
					MathFunction.print("[SCHOOF]: Polynom h(x) berechnet");

					ggT = Polynom.ggT(hx, f_l,p);					// ggT(h(x), f_l);

					if( ( ggT.getDeg() ).compareTo(zero) == 1 )		// deg( ggT(h(x), f_l) ) > 0
					{
						MathFunction.print("[SCHOOF]: deg( ggT(h(x), f_l) ) > 0 ");

						k = Polynom.getK(w,a,b,p,l);				// berechnet das Polynom k
						MathFunction.print("[SCHOOF]: Polynom k berechnet");


						ggT = Polynom.ggT(k, f_l, p);				// ggT(k, f_l);

						if( ( ggT.getDeg() ).compareTo(zero) == 1 )	// deg( ggT(k, f_l) ) > 0
						{
							MathFunction.print("[SCHOOF]: deg(ggT(k, f_"+l+")) > 0" );
							MathFunction.print("[SCHOOF]: t_l = +2w = "+ (two.multiply(w)).mod(l));
						    t_l.set(counter, (two.multiply(w)).mod(l) );	// t_l = +2w mod l
							MathFunction.print("[SCHOOF]:                               t_"+l+" = "+t_l.get(counter)+ " mod "+l);

						} // end if
						else
						{
							MathFunction.print("[SCHOOF]: deg(ggT(k, f_"+l+")) = 0 " );
							MathFunction.print("[SCHOOF]: t_"+l+" = -2w = "+
													((two.multiply(w)).negate()).mod(l) );
							t_l.set(counter, ( (two.multiply(w)).negate() ).mod(l) );// t_"+l+" = -2w mod l
							MathFunction.print("[SCHOOF]:                               t_"+l+" = "+t_l.get(counter)+ " mod "+l);
						} // end else
					} // end if
					else											// deg( ggT(h(x), f_l) ) = 0
					{
						MathFunction.print("[SCHOOF]:                                                 Fall A12");
						t_l.set(counter, zero);						// t_l = 0 mod l
						MathFunction.print("[SCHOOF]:                               t_"+l+" = "+t_l.get(counter)+ " mod "+l);
					} // end else
				} // end if
				else 												// Legendre (pl/l) = -1
				{
					MathFunction.print("[SCHOOF]: Legendre (pl/l) = -1");
					MathFunction.print("[SCHOOF]:                                                 Fall A2");
					t_l.set(counter, zero);							// t_l = 0 mod l
					MathFunction.print("[SCHOOF]:                               t_"+l+" = "+t_l.get(counter)+ " mod "+l);
				} // end else
			} // end if
			else						// deg( ggT(g(x), f_l) ) > 0 (Fall B)
			{
//SCHRITT 6:
				MathFunction.print("[SCHOOF]:                                                 Fall B");

				rl = one;											// rl zu Test fr t_l= {1,..,l-1}

				Polynom Dx1 = Polynom.getDx1(pl, a, b, p, l);
				MathFunction.print("[SCHOOF]: Polynom Dx1 berechnet");

				Polynom Dy1 = Polynom.getDy1(pl, a, b, p, l);
				Dy1 = Polynom.mod(Dy1, f_l, p);
				MathFunction.print("[SCHOOF]: Polynom Dy1 berechnet");

				for(; rl.compareTo(l) == -1; rl = rl.add(one))
				{
					MathFunction.print("[SCHOOF]: rl =" + rl);

					Dx = Polynom.getDx(Dx1,pl,rl,a,b,p,l);				// berechnet das Polynom Dx

					ggT = Polynom.ggT(Dx, f_l, p);					// ggT(Dx, f_l);

					if( ( ggT.getDeg() ).compareTo(zero) == 1 )	// deg( ggT(Dx, f_l) ) > 0
					{
						MathFunction.print("[SCHOOF]: * deg(ggT(Dx, f_"+l+")) > 0 *");

						Dy = Polynom.getDy(Dy1,pl,rl,a,b,p, l);			// berechnet das Polynom Dy

						ggT = Polynom.ggT(Dy, f_l,p);					// ggT(Dy, f_l);

						if( ( ggT.getDeg() ).compareTo(zero) == 1 )	// deg( ggT(Dy, f_l) ) > 0
						{
							MathFunction.print("[SCHOOF]: * deg(ggT(Dy, f_"+l+")) > 0 *");
							MathFunction.print("[SCHOOF]: t_"+l+" = "+ rl);

							t_l.set(counter, rl);
							MathFunction.print("[SCHOOF]:                               t_l = "+t_l.get(counter)+ " mod "+l);

							rl = l;						// dient nur zum Schleifenabbruch
						} // end if
						else
							MathFunction.print("[SCHOOF]:   deg(ggT(Dy, f_"+l+")) = 0");

					} // end if
					else
						MathFunction.print("[SCHOOF]:   deg(ggT(Dx, f_"+l+")) = 0");
				} // end for 2

			}  // end else
		} // end for 1

		MathFunction.print("\n\n==================  ENDE  =======================\n");

//SCHRITT 7: t mit CRS berechnen

		MathFunction.print("[SCHOOF]: berechne t mit dem chinesischen Restsatz");

		t = MathFunction.getCRS(t_l,numberOfPrimes, productOfPrimes);

		temp = (MathFunction.nSqrt(2,p)).multiply(new BigDecimal("2.0"));
		sqrt = MathFunction.floor(temp);

		while( t.compareTo(sqrt) == 1 )				// wenn t > 2 sqrt(p) ist
			t = t.subtract(productOfPrimes);

		while( t.compareTo(sqrt.negate()) == -1 )	// wenn t < -2 sqrt(p) ist
			t = t.add(productOfPrimes);

		MathFunction.print("[SCHOOF]: t = " + t);

//SCHRITT 8:

		MathFunction.print("[SCHOOF]: berechne #E(F_p)");
		MathFunction.print("[SCHOOF]: p + 1 = "+p.add(one));
		order = (p.add(one)).subtract(t);							//#E(F_p) = p + 1 - t
//SCHRITT 9:
		return(order);
	}  // end getOrder_BySchoof

//======================================================================================================
} //EllipticCurve