import java.math.*;

/**
 * <p> Autor Michael Gorski </p>
 * <p> Version 1.0 </p>
 * <p>Copyright: Copyright (c) 2004 </p>
 */

/** Klasse, um Polynome der Form f(x) = a_n X^n + a_n-1 X^n-1 + ... + a_1 X + a_0 darzustellen und
 mit ihnen rechnen zu knnen. */
public class Polynom
{
	final static BigInteger zero = new BigInteger("0");
	final static BigInteger one = new BigInteger("1");
	final static BigInteger two = new BigInteger("2");
	final static BigInteger three = new BigInteger("3");
	final static BigInteger four = new BigInteger("4");
	final static BigInteger five = new BigInteger("5");
	final static BigInteger six = new BigInteger("6");
	final static BigInteger seven = new BigInteger("7");
	final static BigInteger eight = new BigInteger("8");
	final static BigInteger nine = new BigInteger("9");
	final static BigInteger ten = new BigInteger("10");
	final static BigInteger eleven = new BigInteger("11");
	final static BigInteger twelve = new BigInteger("12");

	// Klasse zur Speicherung einer bestimmten Anzahl von Koeffizienten
	class Node
	{
		//public Coeff [] data;
		public BigInteger [] data;
		public BigInteger number;
		public Node next;

		//Node (Coeff [] d, BigInteger num, Node n)
		Node (BigInteger [] d, BigInteger num, Node n)
		{
			number = num;
			data = d;
			next = n;
		}
	}  // end Node
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    final static BigInteger nodeLength	= new BigInteger("40000");	// Anzahl der Elemente pro Knoten
   	BigInteger length = zero;										// Anzahl der Elemente insgesamt
   	BigInteger countNodes = zero;									// Anzahl der Knoten
    BigInteger lastNodeNumber = zero;								// Nummer des letzten Knotens

   	Node start = null;
   	BigInteger deg;													// Grad des Polynoms


//======================================================================================================

	/** Konstruktor zur Erzeugung eines Polynoms vom Grad deg. */
   	public Polynom(BigInteger deg)
   	{
		this.deg = deg;
	   	length = deg.add(one);
	   	countNodes = length.divide(nodeLength);
	   	if ( !((length.mod(nodeLength)).compareTo(zero) == 0) )
	   		countNodes = countNodes.add(one);

		for(BigInteger i = zero; i.compareTo(countNodes) == -1;i = i.add(one))
			insert(new BigInteger[(Polynom.nodeLength).intValue()]);
   	}

//======================================================================================================

   	// Methode fr die Inetialisierung der Polynome
   	private void insert(BigInteger [] o)
   	{

   	   	if (start == null)
   	   	{
   	       	start      = new Node(o, lastNodeNumber, null);
   	       	start.next = start;
	   	}
		else
		{
			start = getNode(lastNodeNumber.subtract(one));
        	start.next = new Node(o, lastNodeNumber, start.next);
		}
		lastNodeNumber = lastNodeNumber.add(one);
   	}  // end insert

//======================================================================================================

	// Methode, um an einen bestimmten Knoten innerhalb der Ringliste zu springen
   	private Node getNode(BigInteger i)
   	{
		while(true)
		{
			if ( (start.number).compareTo(i) == 0 )
				return(start);
			else
			    start = start.next;
		} // end while

   	} //end getNodeNumber

//======================================================================================================
// public Teil																						 ===
//======================================================================================================

	/** ndert einen bestimmten Koeffizienten in dem Polynom. Mit pos
	wird dabei die Position im Polynom angegeben und mit elem der Wert von a_pos.*/
	public void set(BigInteger pos, BigInteger elem)
	{
		// entscheiden in welchem Knoten elem liegt
		BigInteger nodeNumber = pos.divide(nodeLength);
		// entscheiden auf welcher Position im Knoten elem liegt
		BigInteger nodePlace = pos.mod(nodeLength);
		Node n = getNode(nodeNumber);
		n.data[nodePlace.intValue()] = elem;
	} // end set

//======================================================================================================

	/** Gibt den Koeffizienten des Polynoms an Position pos zurck. */
	public BigInteger get(BigInteger pos)
	{
		// entscheiden in welchem Knoten elem liegt
		BigInteger nodeNumber = pos.divide(nodeLength);
		// entscheiden auf welcher Position im Knoten elem liegt
		BigInteger nodePlace = pos.mod(nodeLength);
		Node n = getNode(nodeNumber);

		if ( (n.data[nodePlace.intValue()]) == null )
			return(zero);

		return(n.data[nodePlace.intValue()]);
	} // end get

//======================================================================================================

	/** Gibt das Polynom als formatierten String zurck. */
	public String toString()
	{
		String s = "";
		String s1 = "(DEG: "+deg+"): ";

		BigInteger wert = zero;
		for(BigInteger i = zero; i.compareTo(deg) <= 0;i = i.add(one))
		{
			wert = get(i);
			if (!( (wert).compareTo(zero) == 0))	// wenn a <> 0 ist mu der Koeffizient nicht angezeigt werden
			{

				if(i.compareTo(deg) == 0)			// wenn das letze Element erreicht ist
					s = s + wert;
				else
				{
					if (!( (wert).compareTo(one) == 0))
					{
						if (!( (deg.subtract(i)).compareTo(one) == 0))
							s = s + wert + "X^"+ deg.subtract(i);
						else
							s = s + wert + "X";
					}
					else
					{
						if (!( (deg.subtract(i)).compareTo(one) == 0))
							s = s + "X^"+deg.subtract(i);
						else
							s = s + "X";
					}
					s = s + " + ";
				}
			}
			else if ( ( (wert).compareTo(zero) == 0)&&(i.compareTo(deg) == 0)  )
			{
				if(s.length() < 2)
					s = s + wert;
				else
				  s = s.substring(0,s.length()-2);
			}
			else if ( ( (wert).compareTo(zero) == 0)&&(i.compareTo(zero) == 0)  )
				s = s;
			else
			{
				s = s + " + ";
				if( (i.compareTo(zero) == 1) &&  //wenn mehr als ein Koeffizient vorhanden ist
					(s.length() >= 3)       )
					s = s.substring(0,s.length()-3);
			}

		}  //end for
		s1 = s1 + s;
		return(s1);
	}  // end toString

//======================================================================================================

	/** Gibt den Grad des Polynoms zurck. */
	public BigInteger getDeg()
	{
		return(deg);
	} // end getDeg

//======================================================================================================

	/** Formatiert das Polynom, so dass am Anfang und am Ende keine Koeffizienten stehen, die 0
	sind. */
	public static Polynom prove(Polynom x)
	{
		Polynom a = x;
		BigInteger t = zero;
		BigInteger deg_new = zero;
		for(BigInteger i = zero; i.compareTo(x.getDeg()) <= 0; i = i.add(one))
		{
			if( !( (x.get(i)).compareTo(zero) == 0 ) )
				break;  //beendet Schleife
			else
				t = t.add(one);
		} // end for

		// wenn das Array aufgrund fhrender Nullen zu vereinfachen ist
		if ( t.compareTo(zero) == 1 )
		{
			deg_new = (x.getDeg()).subtract(t);
			if ( deg_new.compareTo(zero) <= 0 )
				a = new Polynom(zero);
			else
			 	a = new Polynom(deg_new);
			for(BigInteger i = t; i.compareTo(x.getDeg()) <= 0; i = i.add(one))
			 	a.set(i.subtract(t), x.get(i) );
		}
		return(a);
	} // end prove

//======================================================================================================

	/** Addiert zwei Polynome und liefert das resultierende Polynom zurck. */
	public static Polynom add(Polynom a, Polynom b, BigInteger p)
	{
		Polynom px = new Polynom(one);
		BigInteger change = zero;

		if( (a.getDeg()).compareTo(b.getDeg()) >= 0 ) //wenn  Grad(a) >= Grad(b) ist
		{
			px = new Polynom(a.getDeg());

			change = (a.getDeg()).subtract(b.getDeg());
			for(BigInteger i = zero; i.compareTo(change) <= 0; i = i.add(one))
				px.set(i,a.get(i));

			// das kleinere Polynom wird durchlaufen und die entsprechenden Stellen addiert
			// hierbei mssen die unterschiedlichen Grade beachtet werden
			for(BigInteger i = zero; i.compareTo(b.getDeg()) <= 0; i = i.add(one))
				px.set(i.add(change), ( ( a.get(i.add(change)) ).add( b.get(i) )	).mod(p) );

		}  // end if
		else if( (b.getDeg()).compareTo(a.getDeg()) == 1 ) //wenn  Grad(b) > Grad(a) ist
		{
			px = new Polynom(b.getDeg());
	        change = (b.getDeg()).subtract(a.getDeg());

			for(BigInteger i = zero; i.compareTo(change) <= 0; i = i.add(one))
				px.set(i,b.get(i));

			// das kleinere Polynom wird durchlaufen und die entsprechenden Stellen addiert
			// hierbei mssen die unterschiedlichen Grade beachtet werden
			for(BigInteger i = zero; i.compareTo(a.getDeg()) <= 0; i = i.add(one))
				px.set(i.add(change),  ( ( b.get(i.add(change)) ).add( a.get(i) )	).mod(p) );
		} // end else
		px = prove(px);
		return(px);
	} // end add

//======================================================================================================

	/** Negiert ein Polynom und liefert dieses zurck. */
	public static Polynom negate(Polynom x, BigInteger p)
	{
		Polynom y = new Polynom(x.getDeg());
		for(BigInteger i = zero; i.compareTo(x.getDeg()) <= 0; i = i.add(one))
			y.set(i, ( (x.get(i)).negate() ).mod(p) );
		return(y);
	}  // end negate

//======================================================================================================

	/** Subtrahiert zwei Polynome voneinander und liefert das resultierende Polynom zurck. */
	public static Polynom sub(Polynom a, Polynom b, BigInteger p)
	{
		Polynom px = new Polynom(one);
		Polynom bneg = negate(b,p);

		if( (a.getDeg()).compareTo(b.getDeg()) >= 0 ) //wenn  Grad(a) >= Grad(b) ist
			px = new Polynom(a.getDeg());

		else if( (b.getDeg()).compareTo(a.getDeg()) == 1 ) //wenn  Grad(b) > Grad(a) ist
			px = new Polynom(b.getDeg());

		px = add(a,bneg,p);
		return(px);
	} // end sub

//======================================================================================================

	/** Multipliziert zwei Polynome miteinander und liefert das resultierende Polynom zurck. */
	public static Polynom mul(Polynom a, Polynom b, BigInteger p)
	{
		BigInteger length = (a.getDeg()).add(b.getDeg());
		Polynom px = new Polynom(length);
		BigInteger c = zero;
		BigInteger d = zero;

		for(BigInteger k = zero; k.compareTo(length) <= 0; k = k.add(one)) // Summe k=0 bis n+m
		{
			c = zero;
			for(BigInteger i = zero; i.compareTo(k) <= 0; i = i.add(one)) // Summe i=0 bis k
			{
				d = k.subtract(i);
				if ( i.compareTo( a.getDeg() ) == 1 ) {}
				else if ( d.compareTo( b.getDeg() ) == 1 ) {}
				else
					c = c.add( (a.get(i)).multiply(b.get(d)) );
			}  // end for
			px.set(k, c.mod(p) );
		} // end for

		px = prove(px);
		return(px);
	} // end mul

//======================================================================================================

    /** Berechnet den Modulo Wert eines Polynoms a mit einer Primzahl p. Dabei werden die Koeffizienten
    a_i modulo p berechnet. */
	public static Polynom mod(Polynom a, BigInteger p)
	{
		for(BigInteger i = zero; i.compareTo(a.getDeg()) <= 0; i = i.add(one))
			a.set(i, (a.get(i)).mod(p) );
		a = prove(a);
		return(a);
	} // end mod

//======================================================================================================

	/** Berechnet die euklidische Division von zwei Polynomen (a/b) und liefert das Ergebnis und
	den Rest in einem Array vom Typ Polynom zurck. An Position 0 steht das Ergebnis und an Position 1
	der Rest.*/
	public static Polynom [] div(Polynom a, Polynom b, BigInteger p)
	{
		Polynom r = a;
		Polynom bnew = b;
		Polynom q = new Polynom(zero);
		Polynom s = new Polynom(zero);
		BigInteger l = zero;
		BigInteger lr = zero;
		BigInteger lb = zero;
		BigInteger sdeg = zero;
		boolean end = false;

		lb = (b.get(zero)).modInverse(p);

		if ( (r.getDeg()).compareTo( b.getDeg() ) == -1 )       // wenn deg(r) < deg(b)
			end = true;
		if ( (r.getDeg()).compareTo( b.getDeg() ) == 0 )		// wenn R = 0
			if (  (r.get(zero)).compareTo(zero) == 0 )
				end = true;

		while ( !end )
		{
			lr = r.get(zero);

			l = (lr).multiply(lb);   							// l(r)*l(b)^-1

			sdeg = (r.getDeg()).subtract(b.getDeg());			// der Grad von S

			s = new Polynom(sdeg);  						// S = l*X^deg(r)-deg(b)
			s.set(zero,l);
			q = add(q,s,p);

			r = sub(r, mul(s,b,p),p);							// R = R - S*B

			if ( (r.getDeg()).compareTo( b.getDeg() ) == -1 )       // wenn deg(r) < deg(b)
				end = true;
			if ( (r.getDeg()).compareTo( b.getDeg() ) == 0 )		// wenn R = 0
				if (  (r.get(zero)).compareTo(zero) == 0 )
					end = true;
		} // end while
        Polynom [] qr = new Polynom [2];
        qr[0] = q;
        qr[1] = r;
		return(qr);
	}  // end div

//======================================================================================================

	/** Berechnet den ggT von zwei Polynomen a und b modulo einer Primzahl p und liefert das
	resultierende Polynom zurck. */
	public static Polynom ggT(Polynom a, Polynom b, BigInteger p)
	{
		Polynom A_new = a;
		Polynom B_new = b;
		Polynom [] qr;
		boolean end = false;

		if ( (B_new.getDeg()).compareTo(zero) == 0 )
			if ( (B_new.get(zero)).compareTo(zero) == 0 )
				end = true;

		while ( !end )
		{

			qr = div(A_new,B_new,p);								// A = BQ + P
			A_new = B_new;											// A = B
			B_new = qr[1];											// B = R

			if ( (B_new.getDeg()).compareTo(zero) == 0 )
				if ( (B_new.get(zero)).compareTo(zero) == 0 )
					end = true;
		}  // end while
		return(A_new);
	} // end ggt

//======================================================================================================

	/** Berechnet das i-te reduzierte Divisionspolynom f_i. */
	public static Polynom getDivisionPolynom(BigInteger i, BigInteger a, BigInteger b, BigInteger p)
	{
		Polynom x = new Polynom(zero);
		BigInteger temp = zero;
		BigInteger m = zero;
		Polynom F = new Polynom(three);									// 4F(x) = 4*x^3 + 4ax +4b
		F.set(zero, (four).mod(p) );
		F.set(two, ( four.multiply(a) ).mod(p)  );
		F.set(three, ( four.multiply(b) ).mod(p)  );
		Polynom f1;
		Polynom f2;
		Polynom f3;
		Polynom f4;
		Polynom f5;
		Polynom f6;
		Polynom f7;
		Polynom f8;

		if ( i.compareTo(new BigInteger("-1")) == 0 )					// i = -1
			x.set(zero, new BigInteger("-1"));
		else if ( i.compareTo(zero) == 0 )								// i = 0
			x.set(zero, zero);
		else if ( i.compareTo(one) == 0 )								// i = 1
			x.set(zero, one );
		else if ( i.compareTo(two) == 0 )								// i = 2
			x.set(zero, one );
		else if ( i.compareTo(three) == 0 )								// i = 3
		{
			x = new Polynom(four);		// f(x) = 3x^4 + 6ax^2 + 12bx - a^2
			x.set(zero, (three).mod(p) );
			x.set(two, (six.multiply(a)).mod(p)  );
			x.set(three, (twelve.multiply(b)).mod(p) );
			x.set(four, ( (a.pow(2)).negate() ).mod(p)  );
			x = prove(x);
		}
		else if ( i.compareTo(four) == 0 )								// i = 4
		{

			x = new Polynom(six);		// f(x) = 2x^6+10ax^4+40bx^3-10a^2x^2 - 8abx -16b^2 - 2a^3
			x.set(zero, two);
			x.set(two, ( ten.multiply(a) ).mod(p)  );
			x.set(three, ( (new BigInteger("40")).multiply(b) ).mod(p)  );
			x.set(four, (  ( ten.multiply( a.pow(2)) ).negate()  ).mod(p)  );

  	        x.set(five, (  ( eight.multiply(a.multiply(b)) ).negate()  ).mod(p)  );

			temp = (  ( (a.pow(3)).multiply(two) ).add( (new BigInteger("16")).multiply(b.pow(2)) )
			       ).negate();

			x.set(six,(temp).mod(p) );
			x = prove(x);
		}
		else if (  (i.mod(two)).compareTo(one) == 0  ) 		// wenn i ungerade
		{
			m = (i.subtract(one)).divide(two);
			if (  ( (m.mod(two)).compareTo(one) == 0 )&& 	// wenn m ungerade und m >= 3 Fall (1)
				  ( m.compareTo(three) >= 0 )  )
			{
				f1 = getDivisionPolynom(m.add(two),a,b,p);				// f1 = f_m+2
				f2 = getDivisionPolynom(m,a,b,p);						// f2 = f^3_m
				f2 = mul(f2, mul(f2,f2,p) ,p);
				f3 = mul(F,F,p);										// f3 = F^2
				f4 = getDivisionPolynom(m.subtract(one),a,b,p);			// f4 = f_m-1
				f5 = getDivisionPolynom(m.add(one),a,b,p);				// f5 = f^3_m+1
				f5 = mul(f5, mul(f5,f5,p), p);

				f6 = mul(f1,f2,p );
				f7 = mul(f3, mul(f4,f5,p), p);
				x = sub(f6,f7,p);
			}  //end if
			else if ( m.compareTo(two) >= 0 ) 				// wenn m gerade und m >= 2  Fall (2)
			{
				f1 = mul(F,F,p);										// f1 = F^2
				f2 = getDivisionPolynom(m.add(two),a,b,p);				// f2 = f_m+2
				f3 = getDivisionPolynom(m,a,b,p);						// f3 = f^3_m
				f3 = mul(f3,mul(f3,f3,p), p);
				f4 = getDivisionPolynom(m.subtract(one),a,b,p);			// f4 = f_m-1
				f5 = getDivisionPolynom(m.add(one),a,b,p);				// f5 = f^3_m+1
				f5 = mul(f5,mul(f5,f5,p), p);

				f6 = mul(f1, mul(f2,f3,p), p );
				f7 = mul(f4,f5,p);
				x = sub(f6,f7,p);
			}  // end else
		} // end else
		else if ( (i.mod(two)).compareTo(zero) == 0  )		// wenn i gerade      Fall (3)
		{
			m = i.divide(two);
			if ( m.compareTo(two) >= 0 )     				// wenn m >= 2
			{
				f1 = getDivisionPolynom(m.add(two),a,b,p);					// f1 = f_m+2
				f2 = getDivisionPolynom(m.subtract(one),a,b,p);				// f2 = f^2_m-1
				f2 = mul(f2,f2,p);
				f3 = getDivisionPolynom(m.subtract(two),a,b,p);				// f3 = f_m-2
				f4 = getDivisionPolynom(m.add(one),a,b,p);					// f4 = f^2_m+1
				f4 = mul(f4,f4,p);
				f5 = getDivisionPolynom(m,a,b,p);							// f5 = f_m

				f6 = mul(f1,f2,p);
				f7 = mul(f3,f4,p);
				f8 = sub(f6,f7,p);
				x = mul(f8,f5,p);
			} // end if
		} // end else
		return(x);
	} // end getDivisionPolynom

//======================================================================================================

	/** Berechnet ein Polynom x hoch einem Skalar a modulo einer Primzahl p. */
	public static Polynom pow(Polynom x, BigInteger a, BigInteger p)
	{
		Polynom y = new Polynom(zero);
		y = add(y,x,p);
		for(BigInteger i = two; i.compareTo(a) <= 0; i = i.add(one))
			y = mul(y,x,p);
		return(y);
	}  // end pow

//======================================================================================================

	/** Berechnet das Produkt eines Skalars n und eines Polynoms x modulo einer Primzahl p. */
	public static Polynom mul(BigInteger n, Polynom x, BigInteger p)
	{
		Polynom y = new Polynom(zero);
		y = add(y,x,p);
		BigInteger deg = x.getDeg();

		for(BigInteger i = zero; i.compareTo(deg) <= 0; i = i.add(one))
			y.set(i, (x.get(i)).multiply(n) );
		return(y);
	}  // end mul

//======================================================================================================

	// Berechnet n*(f_i)^s mod p
	private static Polynom getPolynom(BigInteger n, BigInteger i, BigInteger s,
	                                 BigInteger a, BigInteger b, BigInteger p, BigInteger l)
	{
		Polynom f_l = getDivisionPolynom(l, a, b, p);

		Polynom polynom = mod( getDivisionPolynom(i, a, b, p), f_l, p);

		polynom = mod( pow(polynom, s, p), f_l, p);

		polynom = mul( n.mod(p), polynom, p);

		polynom = mod(polynom, f_l, p);

		return(polynom);
	}  // end getPolynom

//======================================================================================================

	/** Berechnet Polynom poly1 modulo Polynom poly2. */
	public static Polynom mod(Polynom poly1, Polynom poly2 ,BigInteger p)
	{
		Polynom temp = div(poly1, poly2, p) [1];
		return(temp);
	} // end mod

//======================================================================================================

	/** Berechnet das Polynom g(x) oder das Polynom h(x). Der Wert fr type kann entweder "gx" oder
	"hx" sein, daraus wird das entsprechende Polynom berechnet. */
	public static Polynom getGxHx(String type, BigInteger pl, BigInteger a, BigInteger b,
								  BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);

		Polynom f_l = getDivisionPolynom(l, a, b, p);								// f_l

		Polynom poly1;

		if ( type == "gx")
		{

			BigInteger temp = p.pow(2);
			poly1 = new Polynom(temp);								// (x^{p^2} - x )
			poly1.set(zero,one);
			poly1.set(temp.subtract(one), new BigInteger("-1"));
		}
		else // ( type == "hx")
		{
			poly1 = new Polynom(p);									// (x^p - x )
			poly1.set(zero,one);
			poly1.set(p.subtract(one), new BigInteger("-1"));
		}

		Polynom poly2 = pow(getDivisionPolynom(pl, a, b, p), two, p);	// f_pl^2
		poly2 = mod(poly2, f_l, p);

		Polynom poly3 = getDivisionPolynom(pl.subtract(one), a, b, p);  // f_{pl-1}^2
		poly3 = mod(poly3, f_l, p);

		Polynom poly4 = getDivisionPolynom(pl.add(one), a, b, p);  		// f_{pl+1}^2
		poly4 = mod(poly4, f_l, p);

		Polynom poly5 = mul(poly3, poly4, p);
		poly5 = mod(poly5, f_l, p);

		Polynom poly_temp = mul(four, E, p);
		poly_temp = mod(poly_temp, f_l, p);


		if ( (pl.mod(two)).compareTo(zero) == 0 )					// wenn pl = 0 mod 2 (gerade)
		{
			poly2 = mul( poly_temp , poly2, p);					// 4 E f_pl^2
			poly2 = mod(poly2, f_l, p);

		} // end if
		else														// wenn pl = 1 mod 2 (ungerade)
		{
			poly5 = mul( poly_temp, poly5, p);					// 4 E f_{pl-1}f_{pl+1}
			poly5 = mod(poly5, f_l, p);
		} // end else

		polynom = add( mul(poly1, poly2, p) , poly5, p);

		return(polynom);
	}  // end getGxHx

//======================================================================================================

	/** Berechnet das Polynom k.*/
	public static Polynom getK(BigInteger w, BigInteger a, BigInteger b, BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);
		BigInteger temp;

		Polynom f_l = getDivisionPolynom(l, a, b, p);									// f_l

		Polynom poly1 = pow( getDivisionPolynom(w, a, b, p), three, p);  				// f_w^3
		poly1 = mod(poly1, f_l, p);

		Polynom poly_temp;

		Polynom poly2 = getDivisionPolynom(w.add(two), a, b, p);						// f_{w+2}
		poly2 = mod(poly2, f_l, p);

		Polynom poly3 = pow( getDivisionPolynom(w.subtract(one), a, b, p), two, p);		// f_{w-1}^2
		poly3 = mod(poly3, f_l, p);

		Polynom poly4 = getDivisionPolynom(w.subtract(two), a, b, p);					// f_{w-2}
		poly4 = mod(poly4, f_l, p);

		Polynom poly5 = pow( getDivisionPolynom(w.add(one), a, b, p), two, p);			// f_{w+1}^2
		poly5 = mod(poly5, f_l, p);

		Polynom poly6 = mul(poly2, poly3, p);
		poly6 = mod(poly6, f_l, p);

		Polynom poly7 = mul(poly4, poly5, p);
		poly7 = mod(poly7, f_l, p);

		if ( (w.mod(two)).compareTo(zero) == 0 )					// wenn w = 0 mod 2 (gerade)
		{
			temp = (p.add(three)).divide(two);
			poly_temp = mul( new BigInteger(""+16), pow(E, temp, p), p);
			poly1 = mul(poly_temp, poly1, p);						// 16 E^{(p+3)/2 } f_w^3

		} // end if
		else														// wenn w = 1 mod 2 (ungerade)
		{
			temp = (p.subtract(one)).divide(two);
			poly_temp = pow(E, temp, p);
			poly1 = mul(poly_temp, poly1, p);						// E^{(p-1)/2 } f_w^3

		} // end else
		poly1 = mod(poly1, f_l, p);

		polynom = add( sub(poly1, poly6, p), poly7, p);
		polynom = mod(polynom, f_l, p);

		return(polynom);
	} // end getK

//======================================================================================================

	/** Berechnet alpha. */
	public static Polynom getAlpha(BigInteger pl, BigInteger a, BigInteger b, BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);

		Polynom f_l = getDivisionPolynom(l, a, b, p);								// f_l

		Polynom poly1 = getDivisionPolynom(pl.add(two), a, b, p);					// f_{pl+2}
		poly1 = mod(poly1, f_l, p);

		Polynom poly2 = pow(getDivisionPolynom(pl.subtract(one), a, b, p), two, p); // f_{pl-1}^2
		poly2 = mod(poly2, f_l, p);

		Polynom poly3 = getDivisionPolynom(pl.subtract(two), a, b, p);				// f_{pl-2}
		poly3 = mod(poly3, f_l, p);

		Polynom poly4 = pow(getDivisionPolynom(pl.add(one), a, b, p), two, p); 		// f_{pl+1}^2
		poly4 = mod(poly4, f_l, p);

		Polynom poly5 = pow(getDivisionPolynom(pl, a, b, p), three, p); 			// f_{pl}^3
		poly5 = mod(poly5, f_l, p);

		Polynom poly6 = mul(poly1, poly2, p);
		poly6 = mod(poly6, f_l, p);

		Polynom poly7 = mul(poly3, poly4, p);
		poly7 = mod(poly7, f_l, p);

		Polynom poly_temp, poly_temp2;
		BigInteger temp;

		if ( (pl.mod(two)).compareTo(zero) == 0 )				// wenn pl = 0 mod 2 (gerade)
		{
			temp = ( (p.pow(2)).add(three) ).divide(two);						// (p^2+3)/2

			poly_temp = mul(new BigInteger("16"), pow(E, temp, p), p);			// 16 E^((p^2+3)/2)

			poly5 = mul(poly_temp, poly5, p);							// 16 E^((p^2+3)/2) f_{pl}^3
			poly5 = mod(poly5, f_l, p);

			polynom = sub( sub(poly6, poly7, p), poly5, p);
			polynom = mod(polynom, f_l, p);

		} // end if
		else													// wenn pl = 1 mod 2 (ungerade)
		{
			temp = ( (p.pow(2)).subtract(one) ).divide(two);						// (p^2-1)/2

			poly_temp = pow(E, temp, p);										// E^((p^2-1)/2)

			poly5 = mul(poly_temp, poly5, p);							// E^((p^2-1)/2) f_{pl}^3
			poly5 = mod(poly5, f_l, p);

			polynom = sub( sub(poly6, poly7, p), poly5, p);
			polynom = mod(polynom, f_l, p);

			poly_temp2 = mul(four, E, p);										// 4E

			polynom = mul(poly_temp2, polynom, p);
			polynom = mod(polynom, f_l, p);

		} // end else

		return(polynom);
	} // end getAlpha

//======================================================================================================

	/** Berechnet beta. */
	public static Polynom getBeta(BigInteger pl, BigInteger a, BigInteger b, BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);

		Polynom poly_temp;

		Polynom f_l = getDivisionPolynom(l, a, b, p);						// f_l

		Polynom poly1 = getDivisionPolynom(pl, a, b, p);					// f_{pl}

		BigInteger temp = p.pow(2);
		Polynom poly2 = new Polynom(temp);									// (x - x^{p^2} )
		poly2.set(zero,new BigInteger("-1"));
		poly2.set(temp.subtract(one), one);

		poly2 = mod(poly2, f_l, p);

		Polynom poly3 = pow( getDivisionPolynom(pl, a, b, p), two, p);		// f_{pl}^2

		Polynom poly4 = getDivisionPolynom(pl.subtract(one), a, b, p);		// f_{pl-1}

		Polynom poly5 = getDivisionPolynom(pl.add(one), a, b, p);			// f_{pl+1}
		poly5 = mod(poly5, f_l, p);


		if ( (pl.mod(two)).compareTo(zero) == 0 )				// wenn pl = 0 mod 2 (gerade)
		{
			poly_temp = mul(eight, E, p);									// 8E
			poly1 = mul(poly1, poly_temp, p);								// 8E f_{pl}

			poly_temp = mul(four, E, p);									// 4E
			poly3 = mul(poly3, poly_temp, p);								// 4E f_{pl}^2



		} // end if
		else													// wenn pl = 1 mod 2 (ungerade)
		{
			poly_temp = mul(four, E, p); 									// 4E
			poly4 = mul(poly4, poly_temp, p);								// 4E f_{pl-1}

		} // end else

		poly1 = mod(poly1, f_l, p);
		poly3 = mod(poly3, f_l, p);
		poly4 = mod(poly4, f_l, p);

		Polynom poly6 = mul(poly2, poly3, p);
		poly6 = mod(poly6, f_l, p);

		Polynom poly7 = mul(poly4, poly5, p);
		poly7 = mod(poly7, f_l, p);

		Polynom poly8 = sub(poly6, poly7, p);
		poly8 = mod(poly8, f_l, p);

		polynom = mul(poly1, poly8, p);
		polynom = mod(polynom, f_l, p);

		return(polynom);
	} // end getBeta

//======================================================================================================

	/** Berechnet das Polynom Dx1. */
	public static Polynom getDx1(BigInteger pl, BigInteger a, BigInteger b, BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);

		Polynom poly_temp, poly_temp2;

		BigInteger temp;

		Polynom f_l = getDivisionPolynom(l, a, b, p);								// f_l


		Polynom poly1 = getPolynom(one, pl.subtract(one), one, a, b, p, l);			// f_{pl-1}

		Polynom poly2 = getPolynom(one, pl.add(one), one, a, b, p, l);				// f_{pl+1}

		Polynom poly3 = getPolynom(one, pl, two, a, b, p, l);						// f_{pl}^2


		temp = p.pow(2);
		Polynom poly4 = new Polynom(temp);									// (x^{p^2} + x^p + x )
		poly4.set(zero,one);
		poly4.set(temp.subtract(p),one);
		poly4.set(temp.subtract(one), one);

		poly4 = mod(poly4, f_l, p);

/*
		poly_temp = new Polynom(one);
		poly_temp.set(one,one);							 	// temp = x (soll es jedenfalls sein)

		poly4 = Polynom.powmod(poly_temp,p,fl,p); 			// poly4 = x^p mod fl,p
		poly4 = poly4.add(poly_temp);                		// poly4 = x^p + x mod (fl,p)
		poly4 = Polynom.powmod(poly4,p,fl,p);   		// poly4 = (x^p + x)^p mod (fl,p)
         			                               		//   = x^(p^2) + x^p mod (fl,p)
		poly4.add(temp);                    			// poly4 = x^(p^2)+x^p+x mod (fl,p)
*/

		Polynom poly5 = pow( getBeta(pl, a, b, p, l), two, p);							// beta^2

		poly5 = mod(poly5, f_l, p);

		Polynom poly6 = mul(poly3, pow( getAlpha(pl, a, b, p, l), two, p), p);  	//f_{pl}^2 alpha^2

		poly6 = mod(poly6, f_l, p);


		if ( (pl.mod(two)).compareTo(zero) == 0 )				// wenn pl = 0 mod 2 (gerade)
		{
			poly_temp = mul(four, E, p);										// 4E
			poly3 = mul(poly3, poly_temp, p);									// 4E f_{pl}^2

			poly_temp2 = mul( (new BigInteger("16")).mod(p), pow(E, two, p), p);	// 16 E^2

			poly6 = mul(poly_temp2, poly6, p);									// 16 E^2 f_{pl}^2 alpha^2

		} // end if
		else													// wenn pl = 1 mod 2 (ungerade)
		{
			poly_temp = mul(four, E, p);										// 4E
			poly1 = mul(poly_temp, poly1, p);									// 4E f_{pl-1}

			//Test

			poly_temp = mul(new BigInteger("16"), E, p);						// 16E
			poly5 = mul(poly_temp, poly5, p);									// 16E beta^2
			//Test

		} // end else

		poly1 = mod(poly1, f_l, p);
		poly3 = mod(poly3, f_l, p);
		poly6 = mod(poly6, f_l, p);

		Polynom poly7 = mul(poly1, poly2, p);
		poly7 = mod(poly7, f_l, p);

		Polynom poly8 = mul(poly3, poly4, p);
		poly8 = mod(poly8, f_l, p);

		Polynom poly9 = sub(poly7, poly8, p);
		poly9 = mod(poly9, f_l, p);

		Polynom poly10 = mul(poly9, poly5, p);
		poly10 = mod(poly10, f_l, p);

		polynom = add(poly10, poly6, p);
		polynom = mod(polynom, f_l, p);

		return(polynom);
	} // end getDx1

//======================================================================================================

	/** Berechnet das Polynom Dx. */
	public static Polynom getDx(Polynom Dx1, BigInteger pl, BigInteger tl, BigInteger a,
	 							BigInteger b, BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);

		Polynom poly_temp;

		Polynom f_l = getDivisionPolynom(l, a, b, p);

		BigInteger temp = p.multiply(two);											// 2p

		Polynom poly1 = pow( getDivisionPolynom(tl, a, b, p), temp, p);				// f_{tl}^{2p}

		Polynom poly2;

		Polynom poly3 = pow( getDivisionPolynom(tl.subtract(one), a, b, p), p, p);	// f_{tl-1}^p

		Polynom poly4 = pow( getDivisionPolynom(tl.add(one), a, b, p), p, p);		// f_{tl+1}^p
		poly4 = mod(poly4, f_l, p);

		Polynom poly5 = pow( getDivisionPolynom(pl, a, b, p), two, p);				// f_{pl}^2
		poly5 = mod(poly5, f_l, p);

		poly_temp = pow( getBeta(pl, a, b, p, l), two, p);								//beta^2
		poly_temp = mod(poly_temp, f_l, p);

		poly5 = mul(poly_temp, poly5, p);											//beta^2 f_{pl}^2
		poly5 = mod(poly5, f_l, p);


		if ( (pl.mod(two)).compareTo(zero) == 0 )				// wenn pl = 0 mod 2 (gerade)
		{
			//MathFunction.print("[getDx]: pl gerade");

			if ( (tl.mod(two)).compareTo(zero) == 0 )				// wenn tl = 0 mod 2 (gerade)
			{
				//MathFunction.print("[getDx]: tl gerade");

				poly_temp = mul(four.modPow(p,p), pow(E, p, p), p);			// 4^p E^p
				poly1 = mul(poly_temp, poly1, p);							// 4^p E^p f_{tl}^{2p}

				poly2 = mul(four, E, p);									// 4E

			}
			else													// wenn tl = 1 mod 2 (ungerade)
			{
				//MathFunction.print("[getDx]: tl ungerade");

				temp = p.add(one);											// p+1
				poly2 = mul( four.modPow(temp, p), pow(E, temp, p), p);		//4^{p+1} E^{p+1}

			}

		} // end if
		else													// wenn pl = 1 mod 2 (ungerade)
		{
			//MathFunction.print("[getDx]: pl ungerade");

			if ( (tl.mod(two)).compareTo(zero) == 0 )				// wenn tl = 0 mod 2 (gerade)
			{
				//MathFunction.print("[getDx]: tl gerade");

				poly_temp = mul(four.modPow(p,p), pow(E, p, p), p);			// 4^p E^p
				poly1 = mul(poly_temp, poly1, p);							// 4^p E^p f_{tl}^{2p}

				poly2 = mul(new BigInteger("16"), E, p);					// 16E

			}
			else													// wenn tl = 1 mod 2 (ungerade)
			{
			//	MathFunction.print("[getDx]: tl ungerade");

				temp = p.add(one);											// p+1
				poly2 = mul( four.modPow(temp.add(one), p), pow(E, temp, p), p); // //4^{p+2} E^{p+1}

			}

		} // end else

		poly1 = mod(poly1, f_l, p);
		poly2 = mod(poly2, f_l, p);
		poly3 = mod(poly3, f_l, p);

		Polynom poly6 = mul(poly2, poly3, p);
		poly6 = mod(poly6, f_l, p);

		Polynom poly7 = mul(poly4, poly5, p);
		poly7 = mod(poly7, f_l, p);

		Polynom poly8 = mul(poly6, poly7, p);
		poly8 = mod(poly8, f_l, p);

		Polynom poly9 = mul(Dx1, poly1, p);
		poly9 = mod(poly9, f_l, p);

		polynom = add(poly9, poly8, p);
		polynom = mod(polynom, f_l, p);

		return(polynom);
	} // end getDx

//======================================================================================================

	/** Berechnet das Polynom Dy1. */
	public static Polynom getDy1(BigInteger pl, BigInteger a, BigInteger b, BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);

		Polynom poly_temp;

		Polynom f_l = getDivisionPolynom(l, a, b, p);

		poly_temp = pow( getBeta(pl, a, b, p, l), two, p); 				// beta^2
		poly_temp = mod(poly_temp, f_l, p);

		Polynom poly1 = mul( getAlpha(pl, a, b, p, l), poly_temp, p);		// alpha beta^2
		poly1 = mod(poly1, f_l, p);

		BigInteger temp = p.pow(2);
		Polynom poly2 = new Polynom(temp);								// (2 x^{p^2} + x)
		poly2.set(zero,two);
		poly2.set(temp.subtract(one), one);

		poly2 = mod(poly2, f_l, p);

		Polynom poly3 = pow( getDivisionPolynom(pl, a, b, p), two, p); 	// f_{pl}^2
		poly3 = mod(poly3, f_l, p);

		Polynom poly4 = getDivisionPolynom(pl.subtract(one), a, b, p); 	// f_{pl-1}
		poly4 = mod(poly4, f_l, p);

		Polynom poly5 = getDivisionPolynom(pl.add(one), a, b, p); 		// f_{pl+1}
		poly5 = mod(poly5, f_l, p);

		Polynom poly6 = pow( getBeta(pl, a, b, p, l), three, p);			// beta^3
		poly6 = mod(poly6, f_l, p);

		Polynom poly7 = pow( getDivisionPolynom(pl, a, b, p), two, p); 	// f_{pl}^2
		poly7 = mod(poly7, f_l, p);

		Polynom poly8 = pow( getAlpha(pl, a, b, p, l), three, p);			// alpha^3
		poly8 = mod(poly8, f_l, p);

		if ( (pl.mod(two)).compareTo(zero) == 0 )				// wenn pl = 0 mod 2 (gerade)
		{
			poly_temp = mul( four, E, p);  								// 4E
			poly3 = mul(poly_temp, poly3, p);							// 4E f_{pl}^2
			poly3 = mod(poly3, f_l, p);

			temp = ( (p.pow(2)).add(one) ).divide(two);					// (p^2 +1 ) /2
			poly_temp = pow(E, temp, p);								// E^{(p^2 +1 )/2}

			poly6 = mul( mul( two, poly_temp, p), poly6, p);			// 2E^{(p^2 +1 )/2} beta^3
			poly6 = mod(poly6, f_l, p);

			poly_temp = pow(E, two, p);									// E^2

			poly8 = mul( mul( new BigInteger("16"), poly_temp, p), poly8, p); // 16E^2 alpha^3
			poly8 = mod(poly8, f_l, p);

		} // end if
		else													// wenn pl = 1 mod 2 (ungerade)
		{
			poly_temp = mul( new BigInteger("16"), E, p);				// 16E
			poly1 = mul(poly_temp, poly1, p);							// 16E alpha beta^2

			poly_temp = mul( four, E, p);  								// 4E f_{pl-1}
			poly4 = mul(poly_temp, poly4, p);
			poly4 = mod(poly4, f_l, p);

			temp = ( (p.pow(2)).add(three) ).divide(two);				// (p^2 + 3 ) /2
			poly_temp = pow(E, temp, p);								// E^{(p^2 + 3 )/2}
			poly_temp = mod(poly_temp, f_l, p);

			poly6 = mul( mul( new BigInteger("64"), poly_temp, p), poly6, p);// 64E^{(p^2 + 3 )/2} beta^3
			poly6 = mod(poly6, f_l, p);
		} // end else

		Polynom poly9 = mul(poly2, poly3, p);
		poly9 = mod(poly9, f_l, p);

		Polynom poly10 = mul(poly4, poly5, p);
		poly10 = mod(poly10, f_l, p);

		Polynom poly11 = sub(poly9, poly10, p);
		poly11 = mod(poly11, f_l, p);

		Polynom poly12 = mul(poly1, poly11, p);
		poly12 = mod(poly12, f_l, p);

		Polynom poly13 = mul(poly6, poly7, p);
		poly13 = mod(poly13, f_l, p);

		Polynom poly14 = mul(poly8, poly7, p);
		poly14 = mod(poly14, f_l, p);

		Polynom poly15 = sub(poly12, poly13, p);
		poly15 = mod(poly15, f_l, p);

		polynom = sub(poly15, poly14, p);
		polynom = mod(polynom, f_l, p);

		return(polynom);
	} // end getDy1

//======================================================================================================

	/* Berechnet das Polynom Dy. */
	public static Polynom getDy(Polynom Dy1, BigInteger pl, BigInteger tl, BigInteger a, BigInteger b,
								BigInteger p, BigInteger l)
	{
   		Polynom E = new Polynom(three);				// E = x^3 + ax + b
		E.set(zero, one);
		E.set(two, a);
		E.set(three, b);

		Polynom polynom = new Polynom(zero);

		Polynom f_l = getDivisionPolynom(l, a, b, p);

		Polynom poly_temp;

		BigInteger temp;

		Polynom poly1 = null;

		temp = three.multiply(p);

		Polynom poly2 = pow( getDivisionPolynom(tl, a, b, p), temp, p); 	//f_{tl}^{3p}
		poly2 = mod(poly2, f_l, p);

		Polynom poly3 = pow( getBeta(pl, a, b, p, l), three, p);				// beta^3
		poly3 = mod(poly3, f_l, p);

		Polynom poly4 = pow( getDivisionPolynom(pl, a, b, p), two, p); 		//f_{pl}^2
		poly4 = mod(poly4, f_l, p);

		Polynom poly5 = getDivisionPolynom(tl.add(two), a, b, p); 			//f_{tl+2}
		poly5 = mod(poly5, f_l, p);

		Polynom poly6 = pow( getDivisionPolynom(tl.subtract(one), a, b, p), two, p); //f_{tl-1}^2
		poly6 = mod(poly6, f_l, p);

		Polynom poly7 = getDivisionPolynom(tl.subtract(two), a, b, p); 		//f_{tl-2}
		poly7 = mod(poly7, f_l, p);

		Polynom poly8 = pow( getDivisionPolynom(tl.add(one), a, b, p), two, p); 	 //f_{tl+1}^2
		poly8 = mod(poly8, f_l, p);


		if ( (pl.mod(two)).compareTo(zero) == 0 )				// wenn pl = 0 mod 2 (gerade)
		{

			if ( (tl.mod(two)).compareTo(zero) == 0 )				// wenn tl = 0 mod 2 (gerade)
			{
				temp = two.multiply(p);										// 2p

				poly_temp = pow(E, temp, p);								// E^{2p}

				temp = (five.multiply(p)).add(one);							// 5p+1

				poly1 = mul( two.modPow(temp,p), poly_temp, p);				// 2^{5p+1} E^{2p}

				temp = ( p.add(one) ).divide(two);							// (p+1)/2

				poly_temp = mul( pow(E, temp, p), poly3, p);				// E^{(p+1)/2} beta^3

				temp = p.add(two);											// p+2

				poly3 =  mul( two.modPow(temp,p), poly_temp, p);			// 2^{p+2} E^{(p+1)/2} beta^3

			}
			else													// wenn tl = 1 mod 2 (ungerade)
			{
				temp = ( p.add(one) ).divide(two);							// (p+1)/2

				poly_temp = pow(E, temp, p);								// E^{(p+1)/2}

				temp = ( two.multiply(p) ).add(one);						// 2p + 1

				poly1 = mul( two.modPow(temp,p), poly_temp , p);    		// 2^{2p + 1} E^{(p+1)/2}

				temp = p.add(one);											// p+1

				poly_temp = mul( pow(E, temp, p), poly3, p);				// E^{p+1} beta^3

				poly3 = mul( four.modPow(temp,p), poly_temp, p);			// E^{p+1} 4^{p+1} beta^3

			}


		} // end if
		else													// wenn pl = 1 mod 2 (ungerade)
		{
			if ( (tl.mod(two)).compareTo(zero) == 0 )				// wenn tl = 0 mod 2 (gerade)
			{
				temp = two.multiply(p);										// 2p

				poly_temp = pow(E, temp, p);								// E^{2p}

				temp = five.multiply(p) ;									// 5p

				poly1 = mul( two.modPow(temp,p), poly_temp, p); 			// 2^{5p} E^{2p}

				temp = ( p.add(three) ).divide(two);						// (p+3)/2

				poly_temp = mul( pow(E, temp, p), poly3, p);				// E^{(p+3)/2} beta^3

				temp = p.add(six);											// p + 6

				poly3 = mul( two.modPow(temp,p), poly_temp, p);			// 2^{p + 6} E^{(p+3)/2} beta^3

			}
			else													// wenn tl = 1 mod 2 (ungerade)
			{
				temp = ( p.subtract(one) ).divide(two);						// (p-1)/2

				poly_temp = pow(E, temp, p);								// E^{(p-1)/2}

				poly1 = mul( four.modPow(p,p), poly_temp, p); 			// 2^p E^{(p-1)/2}

				temp = p.add(one); 											// p + 1

				poly_temp = mul( pow(E, temp, p), poly3, p);				// E^{p+1} beta^3

				temp = p.add(three);										// p + 3

				poly3 = mul( four.modPow(temp,p), poly_temp, p);			// 4^{p + 3} E^{p+1} beta^3

			}

		} // end else
		poly1 = mod(poly1, f_l, p);
		poly3 = mod(poly3, f_l, p);

		Polynom poly9 = mul( poly1, poly2, p);
		poly9 = mod(poly9, f_l, p);

		Polynom poly10 = mul( poly9, Dy1, p);
		poly10 = mod(poly10, f_l, p);

		Polynom poly11 = mul( poly3, poly4, p);
		poly11 = mod(poly11, f_l, p);

		Polynom poly12 = mul( poly5, poly6, p);
		poly12 = mod(poly12, f_l, p);

		Polynom poly13 = mul( poly7, poly8, p);
		poly13 = mod(poly13, f_l, p);

		Polynom poly14 = pow( sub( poly12, poly13, p), p, p);
		poly14 = mod(poly14, f_l, p);

		Polynom poly15 = mul(poly11, poly14, p);
		poly15 = mod(poly15, f_l, p);

		polynom = sub( poly10, poly15, p);
		polynom = mod(polynom, f_l, p);

		return(polynom);
	} // end getDy

//======================================================================================================
} // end Polynom
